# Project:  MapServer
# Purpose:  xUnit style Python mapscript tests of outputFormatObj
# Author:   Sean Gillies, sgillies@frii.com
#
# ===========================================================================
# Copyright (c) 2004, Sean Gillies
#
# Permission is hereby granted, free of charge, to any person obtaining a
# copy of this software and associated documentation files (the "Software"),
# to deal in the Software without restriction, including without limitation
# the rights to use, copy, modify, merge, publish, distribute, sublicense,
# and/or sell copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included
# in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
# OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
# THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
# DEALINGS IN THE SOFTWARE.
# ===========================================================================

import unittest

import mapscript

from .testing import MapTestCase


class OutputFormatTestCase(unittest.TestCase):
    """http://mapserver.gis.umn.edu/bugs/show_bug.cgi?id=511"""

    def testOutputFormatConstructor(self):
        new_format = mapscript.outputFormatObj("GDAL/GTiff", "gtiff")
        # assert new_format.refcount == 1, new_format.refcount
        assert new_format.name == "gtiff"
        assert new_format.mimetype == "image/tiff"


class MapOutputFormatTestCase(MapTestCase):
    def testAppendNewOutputFormat(self):
        """test that a new output format can be created on-the-fly"""
        num = self.map.numoutputformats
        new_format = mapscript.outputFormatObj("GDAL/GTiff", "gtiffx")
        # assert new_format.refcount == 1, new_format.refcount
        self.map.appendOutputFormat(new_format)
        assert self.map.numoutputformats == num + 1
        # assert new_format.refcount == 2, new_format.refcount
        self.map.selectOutputFormat("gtiffx")
        self.map.save("testAppendNewOutputFormat.map")
        self.map.getLayerByName("INLINE-PIXMAP-RGBA").status = mapscript.MS_ON
        imgobj = self.map.draw()
        filename = "testAppendNewOutputFormat.tif"
        imgobj.save(filename)

    def testRemoveOutputFormat(self):
        """testRemoveOutputFormat may fail depending on GD options"""
        num = self.map.numoutputformats
        new_format = mapscript.outputFormatObj("GDAL/GTiff", "gtiffx")
        self.map.appendOutputFormat(new_format)
        assert self.map.numoutputformats == num + 1
        # assert new_format.refcount == 2, new_format.refcount
        assert self.map.removeOutputFormat("gtiffx") == mapscript.MS_SUCCESS
        # assert new_format.refcount == 1, new_format.refcount
        assert self.map.numoutputformats == num
        self.assertRaises(
            mapscript.MapServerError, self.map.selectOutputFormat, "gtiffx"
        )
        self.map.selectOutputFormat("GTiff")
        assert self.map.outputformat.mimetype == "image/tiff"

    def testBuiltInPNG24Format(self):
        """test built in PNG RGB format"""
        self.map.selectOutputFormat("PNG24")
        assert self.map.outputformat.mimetype == "image/png"
        self.map.getLayerByName("INLINE-PIXMAP-RGBA").status = mapscript.MS_ON
        img = self.map.draw()
        assert img.format.mimetype == "image/png"
        filename = "testBuiltInPNG24Format.png"
        img.save(filename)

    def testBuiltInJPEGFormat(self):
        """test built in JPEG format"""
        self.map.selectOutputFormat("JPEG")
        assert self.map.outputformat.mimetype == "image/jpeg"
        self.map.getLayerByName("INLINE-PIXMAP-RGBA").status = mapscript.MS_ON
        img = self.map.draw()
        assert img.format.mimetype == "image/jpeg"
        filename = "testBuiltInJPEGFormat.jpg"
        img.save(filename)

    def testSelectBuiltInJPEGFormat(self):
        """test selection of built-in JPEG format"""
        self.map.selectOutputFormat("JPEG")
        assert self.map.outputformat.mimetype == "image/jpeg"


class UnsupportedFormatTestCase(unittest.TestCase):
    """I (Sean) don't ever configure for PDF, so this is the unsupported
    format."""

    def testCreateUnsupported(self):
        self.assertRaises(mapscript.MapServerError, mapscript.outputFormatObj, "PDF")


if __name__ == "__main__":
    unittest.main()



# Project:  MapServer
# Purpose:  xUnit style Python mapscript tests of map cloning
# Author:   Sean Gillies, sgillies@frii.com
#
# ===========================================================================
# Copyright (c) 2004, Sean Gillies
#
# Permission is hereby granted, free of charge, to any person obtaining a
# copy of this software and associated documentation files (the "Software"),
# to deal in the Software without restriction, including without limitation
# the rights to use, copy, modify, merge, publish, distribute, sublicense,
# and/or sell copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included
# in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
# OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
# THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
# DEALINGS IN THE SOFTWARE.
# ===========================================================================

import unittest
import urllib
from io import BytesIO

import mapscript

from .testing import TEST_IMAGE as test_image
from .testing import MapTestCase

have_image = False

try:
    from PIL import Image

    have_image = True
except ImportError:
    pass


class ImageObjTestCase(unittest.TestCase):
    def testConstructor(self):
        """imageObj constructor works"""
        imgobj = mapscript.imageObj(10, 10)
        assert imgobj.thisown == 1
        assert imgobj.height == 10
        assert imgobj.width == 10

    def testConstructorWithFormat(self):
        """imageObj with an optional driver works"""
        driver = "AGG/PNG"
        format = mapscript.outputFormatObj(driver)
        imgobj = mapscript.imageObj(10, 10, format)
        assert imgobj.thisown == 1
        assert imgobj.format.driver == driver
        assert imgobj.height == 10
        assert imgobj.width == 10

    def xtestConstructorFilename(self):
        """imageObj with a filename works"""
        imgobj = mapscript.imageObj(test_image)
        assert imgobj.thisown == 1
        assert imgobj.height == 200
        assert imgobj.width == 200
        imgobj.save("testConstructorFilename.png")

    def xtestConstructorFilenameDriver(self):
        """imageObj with a filename and a driver works"""
        imgobj = mapscript.imageObj(test_image)
        assert imgobj.thisown == 1
        assert imgobj.height == 200
        assert imgobj.width == 200
        imgobj.save("testConstructorFilenameDriver.png")

    def xtestConstructorStream(self):
        """imageObj with a file stream works"""
        f = open(test_image, "rb")
        imgobj = mapscript.imageObj(f)
        f.close()
        assert imgobj.thisown == 1
        assert imgobj.height == 200
        assert imgobj.width == 200

    def xtestConstructorBytesIO(self):
        """imageObj with a BytesIO works"""
        with open(test_image, "rb") as f:
            data = f.read()
        s = BytesIO(data)
        imgobj = mapscript.imageObj(s)
        assert imgobj.thisown == 1
        assert imgobj.height == 200
        assert imgobj.width == 200

    def xtestConstructorUrlStream(self):
        """imageObj with a URL stream works"""
        url = urllib.urlopen("http://mapserver.org/_static/banner.png")
        imgobj = mapscript.imageObj(url, "AGG/JPEG")
        assert imgobj.thisown == 1
        assert imgobj.height == 68
        assert imgobj.width == 439
        imgobj.save("testConstructorUrlStream.jpg")


class ImageWriteTestCase(MapTestCase):
    def testImageWrite(self):
        """image writes data to an open filehandle"""
        image = self.map.draw()
        assert image.thisown == 1
        filename = "testImageWrite.png"
        fh = open(filename, "wb")
        image.write(fh)
        fh.close()
        if have_image:
            pyimg = Image.open(filename)
            assert pyimg.format == "PNG"
            assert pyimg.size == (200, 200)
            assert pyimg.mode == "RGB"

    def testImageWriteBytesIO(self):
        """image writes data to a BytesIO instance"""
        image = self.map.draw()
        assert image.thisown == 1

        s = BytesIO()
        image.write(s)

        filename = "testImageWriteBytesIO.png"
        with open(filename, "wb") as fh:
            fh.write(s.getvalue())
        if have_image:
            pyimg = Image.open(filename)
            assert pyimg.format == "PNG"
            assert pyimg.size == (200, 200)
            assert pyimg.mode == "RGB"

    def testImageGetBytes(self):
        """image returns bytes"""
        image = self.map.draw()
        assert image.thisown == 1

        s = BytesIO(image.getBytes())

        filename = "testImageGetBytes.png"
        with open(filename, "wb") as fh:
            fh.write(s.getvalue())
        if have_image:
            pyimg = Image.open(filename)
            assert pyimg.format == "PNG"
            assert pyimg.size == (200, 200)
            assert pyimg.mode == "RGB"

    def testSaveWebImage(self):
        """save an image to the imagepath folder and return its URL"""

        # set the imagepath and imageurl on the MAP WEB object
        self.map.web.imagepath = ""
        self.map.web.imageurl = "/output/"

        image = self.map.draw()
        url = image.saveWebImage()
        assert url.startswith("/output/")
        assert url.endswith(".png")

    def testPasteImage(self):
        """save an image to the imagepath folder and return its URL"""

        image1 = self.map.draw()
        image2 = self.map.draw()

        ret = image1.pasteImage(image2, opacity=0.5, dstx=10, dsty=10)
        assert ret == mapscript.MS_SUCCESS

        filename = "testImagePaste.png"
        fh = open(filename, "wb")
        image1.write(fh)
        fh.close()
        if have_image:
            pyimg = Image.open(filename)
            assert pyimg.format == "PNG"
            assert pyimg.size == (200, 200)
            assert pyimg.mode == "RGB"


if __name__ == "__main__":
    unittest.main()



# Project:  MapServer
# Purpose:  xUnit style Python mapscript tests of Map
# Author:   Sean Gillies, sgillies@frii.com
#
# ===========================================================================
# Copyright (c) 2004, Sean Gillies
#
# Permission is hereby granted, free of charge, to any person obtaining a
# copy of this software and associated documentation files (the "Software"),
# to deal in the Software without restriction, including without limitation
# the rights to use, copy, modify, merge, publish, distribute, sublicense,
# and/or sell copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included
# in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
# OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
# THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
# DEALINGS IN THE SOFTWARE.
# ===========================================================================

import unittest

import mapscript

from .testing import TESTMAPFILE, MapTestCase


class MapConstructorTestCase(unittest.TestCase):
    def testMapConstructorNoArg(self):
        """MapConstructorTestCase.testMapConstructorNoArg: test map constructor with no argument"""
        test_map = mapscript.mapObj()
        assert test_map.__class__.__name__ == "mapObj"
        assert test_map.thisown == 1

    def testMapConstructorEmptyStringArg(self):
        """MapConstructorTestCase.testMapConstructorEmptyStringArg:
        test map constructor with old-style empty string argument"""
        test_map = mapscript.mapObj("")
        assert test_map.__class__.__name__ == "mapObj"
        assert test_map.thisown == 1

    def testMapConstructorFilenameArg(self):
        """MapConstructorTestCasetest.testMapConstructorEmptyStringArg: map constructor with filename argument"""
        test_map = mapscript.mapObj(TESTMAPFILE)
        assert test_map.__class__.__name__ == "mapObj"
        assert test_map.thisown == 1


class MapExtentTestCase(MapTestCase):
    def testSetExtent(self):
        """MapExtentTestCase.testSetExtent: test the setting of a mapObj's extent"""
        test_map = mapscript.mapObj(TESTMAPFILE)
        e = test_map.extent
        result = test_map.setExtent(e.minx, e.miny, e.maxx, e.maxy)
        self.assertAlmostEqual(test_map.scaledenom, 14.24445829)
        assert result == mapscript.MS_SUCCESS, result

    def testSetExtentBadly(self):
        """MapExtentTestCase.testSetExtentBadly: test that mapscript raises an error for an invalid mapObj extent"""
        test_map = mapscript.mapObj(TESTMAPFILE)
        self.assertRaises(
            mapscript.MapServerError, test_map.setExtent, 1.0, -2.0, -3.0, 4.0
        )

    def testReBindingExtent(self):
        """test the rebinding of a map's extent"""
        test_map = mapscript.mapObj(TESTMAPFILE)
        rect1 = mapscript.rectObj(-10.0, -10.0, 10.0, 10.0)
        rect2 = mapscript.rectObj(-10.0, -10.0, 10.0, 10.0)
        test_map.extent = rect1
        assert repr(test_map.extent) != repr(rect1), (test_map.extent, rect1)
        del rect1
        self.assertRectsEqual(test_map.extent, rect2)


class MapLayersTestCase(MapTestCase):
    def testMapInsertLayer(self):
        """MapLayersTestCase.testMapInsertLayer: test insertion of a new layer at default (last) index"""
        n = self.map.numlayers
        layer = mapscript.layerObj()
        layer.name = "new"
        assert layer.map is None, layer.map
        index = self.map.insertLayer(layer)
        assert layer.map is not None, layer.map
        assert index == n, index
        assert self.map.numlayers == n + 1
        names = [self.map.getLayer(i).name for i in range(self.map.numlayers)]
        assert names == [
            "RASTER",
            "POLYGON",
            "LINE",
            "POINT",
            "INLINE",
            "INLINE-PIXMAP-RGBA",
            "INLINE-PIXMAP-PCT",
            "new",
        ]
        order = self.map.getLayerOrder()
        assert order == (0, 1, 2, 3, 4, 5, 6, 7), order

    def testMapInsertLayerAtZero(self):
        """MapLayersTestCase.testMapInsertLayerAtZero: test insertion of a new layer at first index"""
        n = self.map.numlayers
        layer = mapscript.layerObj()
        layer.name = "new"
        assert layer.map is None, layer.map
        index = self.map.insertLayer(layer, 0)
        assert layer.map is not None, layer.map
        assert index == 0, index
        assert self.map.numlayers == n + 1
        names = [self.map.getLayer(i).name for i in range(self.map.numlayers)]
        assert names == [
            "new",
            "RASTER",
            "POLYGON",
            "LINE",
            "POINT",
            "INLINE",
            "INLINE-PIXMAP-RGBA",
            "INLINE-PIXMAP-PCT",
        ], names
        order = self.map.getLayerOrder()
        assert order == (0, 1, 2, 3, 4, 5, 6, 7), order

    def testMapInsertLayerDrawingOrder(self):
        """MapLayersTestCase.testMapInsertLayerDrawingOrder: test affect of
        insertion of a new layer at index 1 on drawing order"""
        n = self.map.numlayers
        assert n == 7
        # reverse layer drawing order
        o_start = (6, 5, 4, 3, 2, 1, 0)
        self.map.setLayerOrder(o_start)
        # insert Layer
        layer = mapscript.layerObj()
        layer.name = "new"
        index = self.map.insertLayer(layer, 1)
        assert index == 1, index
        # We expect our new layer to be at index 1 in drawing order as well
        order = self.map.getLayerOrder()
        assert order == (7, 1, 6, 5, 4, 3, 2, 0), order

    def testMapInsertLayerBadIndex(self):
        """MapLayersTestCase.testMapInsertLayerBadIndex: expect an exception when index is too large"""
        layer = mapscript.layerObj()
        self.assertRaises(
            mapscript.MapServerChildError, self.map.insertLayer, layer, 1000
        )

    def testMapInsertNULLLayer(self):
        """expect an exception on attempt to insert a NULL Layer"""
        self.assertRaises(mapscript.MapServerChildError, self.map.insertLayer, None)

    def testMapRemoveLayerAtTail(self):
        """removal of highest index (tail) layer"""
        n = self.map.numlayers
        layer = self.map.removeLayer(n - 1)
        assert self.map.numlayers == n - 1
        assert layer.name == "INLINE-PIXMAP-PCT"
        assert layer.thisown == 1
        del layer
        order = self.map.getLayerOrder()
        assert order == (0, 1, 2, 3, 4, 5), order

    def testMapRemoveLayerAtZero(self):
        """removal of lowest index (0) layer"""
        n = self.map.numlayers
        layer = self.map.removeLayer(0)
        assert self.map.numlayers == n - 1
        assert layer.name == "RASTER"
        order = self.map.getLayerOrder()
        assert order == (0, 1, 2, 3, 4, 5), order

    def testMapRemoveLayerDrawingOrder(self):
        """test affect of layer removal on drawing order"""
        n = self.map.numlayers
        # reverse layer drawing order
        o_start = (6, 5, 4, 3, 2, 1, 0)
        self.map.setLayerOrder(o_start)
        layer = self.map.removeLayer(1)
        assert self.map.numlayers == n - 1
        assert layer.name == "POLYGON"
        order = self.map.getLayerOrder()
        assert order == (5, 4, 3, 2, 1, 0), order
        names = [self.map.getLayer(i).name for i in range(self.map.numlayers)]
        assert names == [
            "RASTER",
            "LINE",
            "POINT",
            "INLINE",
            "INLINE-PIXMAP-RGBA",
            "INLINE-PIXMAP-PCT",
        ], names


class MapExceptionTestCase(MapTestCase):
    def testDrawBadData(self):
        """MapExceptionTestCase.testDrawBadData: a bad data descriptor in a layer returns proper error"""
        self.map.getLayerByName("POLYGON").data = "foo"
        self.assertRaises(mapscript.MapServerError, self.map.draw)


class EmptyMapExceptionTestCase(unittest.TestCase):
    def setUp(self):
        self.map = mapscript.mapObj("")

    def tearDown(self):
        self.map = None

    def testDrawEmptyMap(self):
        """EmptyMapExceptionTestCase.testDrawEmptyMap: drawing an empty map returns proper error"""
        self.assertRaises(mapscript.MapServerError, self.map.draw)


class NoFontSetTestCase(unittest.TestCase):
    def testNoGetFontSetFile(self):
        """an empty map should have fontset filename == None"""
        self.map = mapscript.mapObj()
        assert self.map.fontset.filename is None


class MapFontSetTestCase(MapTestCase):
    def testGetFontSetFile(self):
        """expect fontset file to be 'fonts.txt'"""
        file = self.map.fontset.filename
        assert file == "fonts.txt", file


class MapSizeTestCase(MapTestCase):
    def testDefaultSize(self):
        assert self.map.width == 200
        assert self.map.height == 200

    def testSetSize(self):
        retval = self.map.setSize(480, 480)
        assert retval == mapscript.MS_SUCCESS, retval
        assert self.map.width == 480
        assert self.map.height == 480


class MapSetWKTTestCase(MapTestCase):
    def testOGCWKT(self):
        self.map.setWKTProjection(
            """PROJCS["unnamed",GEOGCS["WGS 84",DATUM["WGS_1984",
                                     SPHEROID["WGS 84",6378137,298.257223563]],
                                     PRIMEM["Greenwich",0],
                                     UNIT["Degree",0.0174532925199433]],
                                     PROJECTION["Albers_Conic_Equal_Area"],
                                     PARAMETER["standard_parallel_1", 65], PARAMETER["standard_parallel_2", 55],
                                     PARAMETER["latitude_of_center", 0], PARAMETER["longitude_of_center", -153],
                                     PARAMETER["false_easting", -4943910.68], PARAMETER["false_northing", 0],
                                     UNIT["metre",1.0]
                                     ]"""
        )
        proj4 = self.map.getProjection()

        assert proj4.find("+proj=aea") != -1
        assert proj4.find("+datum=WGS84") != -1
        assert mapscript.projectionObj(proj4).getUnits() != mapscript.MS_DD

    def testESRIWKT(self):
        self.map.setWKTProjection(
            'ESRI::PROJCS["Pulkovo_1995_GK_Zone_2", GEOGCS["GCS_Pulkovo_1995", '
            'DATUM["D_Pulkovo_1995", SPHEROID["Krasovsky_1940", 6378245, 298.3]], '
            'PRIMEM["Greenwich", 0], '
            'UNIT["Degree", 0.017453292519943295]], PROJECTION["Gauss_Kruger"], '
            'PARAMETER["False_Easting", 2500000], '
            'PARAMETER["False_Northing", 0], PARAMETER["Central_Meridian", 9], '
            'PARAMETER["Scale_Factor", 1], '
            'PARAMETER["Latitude_Of_Origin", 0], UNIT["Meter", 1]]'
        )
        proj4 = self.map.getProjection()

        assert proj4.find("+proj=tmerc") != -1
        assert proj4.find("+ellps=krass") != -1
        assert (mapscript.projectionObj(proj4)).getUnits() != mapscript.MS_DD

    def testWellKnownGEOGCS(self):
        self.map.setWKTProjection("WGS84")
        proj4 = self.map.getProjection()
        assert (
            "+proj=longlat" in proj4 and "+datum=WGS84" in proj4
        ) or proj4 == "+init=epsg:4326", proj4
        assert (mapscript.projectionObj(proj4)).getUnits() != mapscript.MS_METERS


class MapRunSubTestCase(MapTestCase):
    def testDefaultSubstitutions(self):
        s = """
MAP
    WEB
        VALIDATION
            'key1' '.*'
            'default_key1' 'Test Title'
        END
        METADATA
            "wms_title" "%key1%"
        END
    END
END
        """
        map = mapscript.fromstring(s)
        map.applyDefaultSubstitutions()
        assert map.web.metadata["wms_title"] == "Test Title"

    def testRuntimeSubstitutions(self):
        """
        For supported parameters see https://mapserver.org/cgi/runsub.html#parameters-supported
        """
        s = """
MAP
    WEB
        VALIDATION
            'key1' '.*'
            'default_key1' 'Test Title'
        END
        METADATA
            "wms_title" "%key1%"
        END
    END
    LAYER
        TYPE POINT
        FILTER ([size]<%my_filter%)
        VALIDATION
            'my_filter' '^[0-9]$'
        END
    END
END
        """
        map = mapscript.fromstring(s)

        d = {"key1": "New Title", "my_filter": "3"}
        map.applySubstitutions(d)

        assert map.web.metadata["wms_title"] == "New Title"
        assert map.getLayer(0).getFilterString() == "([size]<3)"


class MapSLDTestCase(MapTestCase):
    def test(self):

        test_map = mapscript.mapObj(TESTMAPFILE)
        result = test_map.generateSLD()
        assert result.startswith('<StyledLayerDescriptor version="1.0.0"'), result

        result = test_map.generateSLD("1.1.0")
        assert result.startswith('<StyledLayerDescriptor version="1.1.0"'), result


if __name__ == "__main__":
    unittest.main()



# Project:  MapServer
# Purpose:  xUnit style Python mapscript tests of Map "zooming"
# Author:   Sean Gillies, sgillies@frii.com
#
# ===========================================================================
# Copyright (c) 2004, Sean Gillies
#
# Permission is hereby granted, free of charge, to any person obtaining a
# copy of this software and associated documentation files (the "Software"),
# to deal in the Software without restriction, including without limitation
# the rights to use, copy, modify, merge, publish, distribute, sublicense,
# and/or sell copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included
# in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
# OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
# THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
# DEALINGS IN THE SOFTWARE.
# ===========================================================================

import unittest

import mapscript

from .testing import MapZoomTestCase


class ZoomPointTestCase(MapZoomTestCase):
    def testRecenter(self):
        """ZoomPointTestCase.testRecenter: recentering the map with a point returns the same extent"""
        w, h = (self.mapobj1.width, self.mapobj1.height)
        p = mapscript.pointObj(50.0, 50.0)
        extent = self.mapobj1.extent
        self.mapobj1.zoomPoint(1, p, w, h, extent, None)
        new_extent = self.mapobj1.extent
        self.assertRectsEqual(new_extent, mapscript.rectObj(-50, -50, 50, 50))

    def testZoomInPoint(self):
        """ZoomPointTestCase.testZoomInPoint: zooming in by a power of 2 returns the proper extent"""
        w, h = (self.mapobj1.width, self.mapobj1.height)
        p = mapscript.pointObj(50.0, 50.0)
        extent = self.mapobj1.extent
        self.mapobj1.zoomPoint(2, p, w, h, extent, None)
        new_extent = self.mapobj1.extent
        self.assertRectsEqual(new_extent, mapscript.rectObj(-25, -25, 25, 25))

    def testZoomOutPoint(self):
        """ZoomPointTestCase.testZoomOutPoint: zooming out by a power of 2 returns the proper extent"""
        w, h = (self.mapobj1.width, self.mapobj1.height)
        p = mapscript.pointObj()
        p.x, p.y = (50, 50)
        extent = self.mapobj1.extent
        self.mapobj1.zoomPoint(-2, p, w, h, extent, None)
        new_extent = self.mapobj1.extent
        self.assertRectsEqual(new_extent, mapscript.rectObj(-100, -100, 100, 100))

    def testZoomOutPointConstrained(self):
        """ZoomPointTestCase.testZoomOutPointConstrained: zooming out to a constrained extent returns proper extent"""
        w, h = (self.mapobj1.width, self.mapobj1.height)
        max = mapscript.rectObj()
        max.minx, max.miny, max.maxx, max.maxy = (-100.0, -100.0, 100.0, 100.0)
        p = mapscript.pointObj()
        p.x, p.y = (50, 50)
        extent = self.mapobj1.extent
        self.mapobj1.zoomPoint(-4, p, w, h, extent, max)
        new_extent = self.mapobj1.extent
        self.assertRectsEqual(new_extent, max)

    def testZoomBadSize(self):
        """ZoomPointTestCase.testZoomBadSize: zooming to a bad size raises proper error"""
        p = mapscript.pointObj()
        p.x, p.y = (50, 50)
        extent = self.mapobj1.extent
        w = 0
        h = -1
        self.assertRaises(
            mapscript.MapServerError, self.mapobj1.zoomPoint, -2, p, w, h, extent, None
        )

    def testZoomBadExtent(self):
        """ZoomPointTestCase.testZoomBadExtent: zooming to a bad extent raises proper error"""
        p = mapscript.pointObj()
        p.x, p.y = (50, 50)
        extent = self.mapobj1.extent
        extent.maxx = extent.maxx - 1000000
        w = 100
        h = 100
        self.assertRaises(
            mapscript.MapServerError, self.mapobj1.zoomPoint, -2, p, w, h, extent, None
        )


class ZoomRectangleTestCase(MapZoomTestCase):
    def testZoomRectangle(self):
        """ZoomRectangleTestCase.testZoomRectangle: zooming to an extent returns proper map extent"""
        w, h = (self.mapobj1.width, self.mapobj1.height)
        r = mapscript.rectObj(1, 26, 26, 1, 1)
        extent = self.mapobj1.extent
        self.mapobj1.zoomRectangle(r, w, h, extent, None)
        new_extent = self.mapobj1.extent
        self.assertRectsEqual(new_extent, mapscript.rectObj(-49, 24, -24, 49))

    def testZoomRectangleConstrained(self):
        """ZoomRectangleTestCase.testZoomRectangleConstrained: zooming to a constrained extent returns proper extent"""
        w, h = (self.mapobj1.width, self.mapobj1.height)
        max = mapscript.rectObj(-100.0, -100.0, 100.0, 100.0)
        r = mapscript.rectObj(0, 200, 200, 0, 1)
        extent = self.mapobj1.extent
        self.mapobj1.zoomRectangle(r, w, h, extent, max)
        new_extent = self.mapobj1.extent
        self.assertRectsEqual(new_extent, max)

    def testZoomRectangleBadly(self):
        """zooming into an invalid extent raises proper error"""
        w, h = (self.mapobj1.width, self.mapobj1.height)
        r = mapscript.rectObj(0, 0, 200, 200)
        extent = self.mapobj1.extent
        self.assertRaises(
            mapscript.MapServerError, self.mapobj1.zoomRectangle, r, w, h, extent, None
        )


class ZoomScaleTestCase(MapZoomTestCase):
    def testRecenter(self):
        """ZoomScaleTestCase.testRecenter: recentering map returns proper extent"""
        w, h = (self.mapobj1.width, self.mapobj1.height)  # 100 by 100
        p = mapscript.pointObj()
        p.x, p.y = (50, 50)
        scale = 2834.6472
        extent = self.mapobj1.extent
        self.mapobj1.zoomScale(scale, p, w, h, extent, None)
        assert self.mapobj1.scaledenom == scale
        new_extent = self.mapobj1.extent
        # self.assertRectsEqual(new_extent, mapscript.rectObj(-50, -50, 50, 50))  # old values
        self.assertRectsEqual(new_extent, mapscript.rectObj(-49.5, -49.5, 49.5, 49.5))

    def testZoomInScale(self):
        """ZoomScaleTestCase.testZoomInScale: zooming in to a specified scale returns proper extent"""
        w, h = (self.mapobj1.width, self.mapobj1.height)
        p = mapscript.pointObj()
        p.x, p.y = (50, 50)
        scale = 1417.3236
        extent = self.mapobj1.extent
        self.mapobj1.zoomScale(scale, p, w, h, extent, None)
        new_extent = self.mapobj1.extent
        # self.assertRectsEqual(new_extent, mapscript.rectObj(-25, -25, 25, 25))  # old values
        self.assertRectsEqual(
            new_extent, mapscript.rectObj(-24.75, -24.75, 24.75, 24.75)
        )

    def testZoomOutScale(self):
        """ZoomScaleTestCase.testZoomOutScale: zooming out to a specified scale returns proper extent"""
        w, h = (self.mapobj1.width, self.mapobj1.height)
        p = mapscript.pointObj()
        p.x, p.y = (50, 50)
        scale = 5669.2944
        extent = self.mapobj1.extent
        self.mapobj1.zoomScale(scale, p, w, h, extent, None)
        new_extent = self.mapobj1.extent
        # self.assertRectsEqual(new_extent, mapscript.rectObj(-100, -100, 100, 100))  # old values
        self.assertRectsEqual(new_extent, mapscript.rectObj(-99, -99, 99, 99))

    def testZoomOutPointConstrained(self):
        """ZoomScaleTestCase.testZoomOutPointConstrained: zooming out to a constrained extent returns proper extent"""
        w, h = (self.mapobj1.width, self.mapobj1.height)
        max = mapscript.rectObj()
        max.minx, max.miny, max.maxx, max.maxy = (-100.0, -100.0, 100.0, 100.0)
        p = mapscript.pointObj()
        p.x, p.y = (50, 50)
        extent = self.mapobj1.extent
        scale = 10000
        self.mapobj1.zoomScale(scale, p, w, h, extent, max)
        new_extent = self.mapobj1.extent
        self.assertRectsEqual(new_extent, max)


if __name__ == "__main__":
    unittest.main()



# Project:  MapServer
# Purpose:  xUnit style Python mapscript tests of classObj
# Author:   Sean Gillies, sgillies@frii.com
#
# ===========================================================================
# Copyright (c) 2004, Sean Gillies
#
# Permission is hereby granted, free of charge, to any person obtaining a
# copy of this software and associated documentation files (the "Software"),
# to deal in the Software without restriction, including without limitation
# the rights to use, copy, modify, merge, publish, distribute, sublicense,
# and/or sell copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included
# in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
# OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
# THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
# DEALINGS IN THE SOFTWARE.
# ===========================================================================

import unittest

import mapscript

from .testing import MapTestCase


class ClassObjTestCase(unittest.TestCase):
    def testConstructorNoArg(self):
        c = mapscript.classObj()
        assert c.thisown == 1
        assert c.layer is None
        assert c.numstyles == 0

    def testConstructorWithArg(self):
        lyr = mapscript.layerObj()
        lyr.name = "foo"
        c = mapscript.classObj(lyr)
        assert c.thisown == 1
        assert c.layer.name == lyr.name
        assert c.numstyles == 0

    def testGetSetAttributes(self):
        c = mapscript.classObj()

        val = "/tmp/legend.png"
        c.keyimage = val
        assert c.keyimage == val

        c.debug = mapscript.MS_TRUE
        assert c.debug == mapscript.MS_TRUE

        val = "Group1"
        c.group = val
        assert c.group == val

        val = 10000
        c.maxscaledenom = val
        assert c.maxscaledenom == val

        val = 3
        c.minfeaturesize = val
        assert c.minfeaturesize == val

        val = 1000
        c.minscaledenom = val
        assert c.minscaledenom == val

        val = "Class1"
        c.name = val
        assert c.name == val

        c.status = mapscript.MS_OFF
        assert c.status == mapscript.MS_OFF

        val = "template.html"
        c.template = val
        assert c.template == val

        val = "Title1"
        c.title = val
        assert c.title == val


class ClassCloningTestCase(unittest.TestCase):
    def testCloneClass(self):
        """check attributes of a cloned class"""
        c = mapscript.classObj()
        c.minscaledenom = 5.0
        clone = c.clone()
        assert clone.thisown == 1
        assert clone.minscaledenom == 5.0


class ClassIconTestCase(MapTestCase):

    """testing for bug 1250"""

    def testAlphaTransparentPixmap(self):
        lo = self.map.getLayerByName("INLINE-PIXMAP-RGBA")
        co = lo.getClass(0)
        self.map.selectOutputFormat("PNG")
        im = co.createLegendIcon(self.map, lo, 48, 48)
        im.save("testAlphaTransparentPixmapIcon.png")

    def testAlphaTransparentPixmapPNG24(self):
        lo = self.map.getLayerByName("INLINE-PIXMAP-RGBA")
        co = lo.getClass(0)
        self.map.selectOutputFormat("PNG24")
        im = co.createLegendIcon(self.map, lo, 48, 48)
        im.save("testAlphaTransparentPixmapIcon24.png")

    def testAlphaTransparentPixmapJPG(self):
        lo = self.map.getLayerByName("INLINE-PIXMAP-RGBA")
        co = lo.getClass(0)
        self.map.selectOutputFormat("JPEG")
        im = co.createLegendIcon(self.map, lo, 48, 48)
        im.save("testAlphaTransparentPixmapIcon.jpg")

    def testIndexedTransparentPixmap(self):
        lo = self.map.getLayerByName("INLINE-PIXMAP-PCT")
        lo.type = mapscript.MS_LAYER_POINT
        co = lo.getClass(0)
        self.map.selectOutputFormat("PNG")
        im = co.createLegendIcon(self.map, lo, 32, 32)
        im.save("testIndexedTransparentPixmapIcon.png")

    def testIndexedTransparentPixmapJPG(self):
        lo = self.map.getLayerByName("INLINE-PIXMAP-PCT")
        lo.type = mapscript.MS_LAYER_POINT
        co = lo.getClass(0)
        self.map.selectOutputFormat("JPEG")
        im = co.createLegendIcon(self.map, lo, 32, 32)
        im.save("testIndexedTransparentPixmapIcon.jpg")


if __name__ == "__main__":
    unittest.main()



# Project:  MapServer
# Purpose:  xUnit style Python mapscript tests of Map
# Author:   Umberto Nicoletti, unicoletti@prometeo.it
#
# ===========================================================================
# Copyright (c) 2004, Sean Gillies
#
# Permission is hereby granted, free of charge, to any person obtaining a
# copy of this software and associated documentation files (the "Software"),
# to deal in the Software without restriction, including without limitation
# the rights to use, copy, modify, merge, publish, distribute, sublicense,
# and/or sell copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included
# in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
# OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
# THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
# DEALINGS IN THE SOFTWARE.
# ===========================================================================

import unittest

import mapscript

from .testing import TESTMAPFILE


class ReferenceCountingTestCase(unittest.TestCase):
    def initMap(self):
        self.map = mapscript.mapObj(TESTMAPFILE)

    def testMap(self):
        """ReferenceCountingTestCase.testMap: test map constructor with no argument"""
        test_map = mapscript.mapObj()
        assert test_map.__class__.__name__ == "mapObj"
        assert test_map.thisown == 1
        assert test_map.refcount == 1

    def testMapWithDefaultMap(self):
        """ReferenceCountingTestCase.testTestMap: test map constructor with default map file"""
        test_map = mapscript.mapObj(TESTMAPFILE)
        assert test_map.__class__.__name__ == "mapObj"
        assert test_map.thisown == 1
        assert test_map.refcount == 1
        assert test_map.getLayer(0).refcount == 2

    def testMapLayersCounting(self):
        """ReferenceCountingTestCase.testTestMap: test map constructor with default map file"""
        self.initMap()
        layerRef1 = self.map.getLayer(0)
        assert layerRef1.refcount == 2
        layerRef2 = self.map.getLayer(0)
        assert layerRef2.refcount == 3

    def testMapInsertedLayer(self):
        """MapLayersTestCase.testMapInsertedLayer: test insertion of a new layer at default (last) index"""
        self.initMap()
        n = self.map.numlayers
        layer = mapscript.layerObj()
        layer.name = "new"
        assert layer.refcount == 1
        assert layer.thisown == 1
        index = self.map.insertLayer(layer)
        assert index == n, index
        assert self.map.numlayers == n + 1
        assert layer.refcount == 2
        assert layer.thisown == 1
        assert self.map.getLayer(index).refcount == 3
        assert self.map.getLayer(index).thisown == 1
        assert layer.refcount == 2
        assert layer.thisown == 1

    def testMapInsertedLayerWithIndex(self):
        """MapLayersTestCase.testMapInsertedLayerWithIndex: test insertion of a new layer at index 0"""
        self.initMap()
        n = self.map.numlayers
        layer = mapscript.layerObj()
        layer.name = "new"
        assert layer.refcount == 1
        assert layer.thisown == 1
        index = self.map.insertLayer(layer, 0)
        assert index == 0, index
        assert self.map.numlayers == n + 1
        assert layer.refcount == 2
        assert layer.thisown == 1
        assert self.map.getLayer(index).refcount == 3
        assert self.map.getLayer(index).thisown == 1
        assert layer.refcount == 2
        assert layer.thisown == 1

    def testMapRemovedLayerAtTail(self):
        """removal of highest index (tail) layer"""
        self.initMap()
        n = self.map.numlayers
        layer = self.map.removeLayer(n - 1)
        assert self.map.numlayers == n - 1
        assert layer.name == "INLINE-PIXMAP-PCT"
        assert layer.thisown == 1
        assert layer.refcount == 1, layer.refcount
        self.map.draw()

    def testMapRemovedLayerAtZero(self):
        """removal of lowest index (0) layer"""
        self.initMap()
        n = self.map.numlayers
        layer = self.map.removeLayer(0)
        assert self.map.numlayers == n - 1
        assert layer.name == "RASTER"
        assert layer.thisown == 1
        assert layer.refcount == 1, layer.refcount
        self.map.draw()

    def testBehaveWhenParentIsNull(self):
        """behave when parent (map) is null"""
        self.initMap()
        layer = mapscript.layerObj()
        layer.name = "new"
        index = self.map.insertLayer(layer, 0)
        assert index == 0, index
        self.map = None
        assert layer.map is not None, layer.map
        exception = None
        try:
            layer.open()
        except Exception:
            # must fail because the new layer is missing information
            exception = True
        assert exception is True

    def testLayerClone(self):
        """Clone a layer"""
        layer = mapscript.layerObj()
        layer.name = "sample"
        copy = layer.clone()

        assert layer.refcount == 1, layer.refcount
        assert copy.refcount == 1, copy.refcount
        assert layer.name == copy.name
        assert copy.map is None

    def testMapGetLayerByName(self):
        """behave when parent (map) is null"""
        self.initMap()
        layer = self.map.getLayerByName("RASTER")

        assert layer.refcount == 2, layer.refcount
        assert layer.thisown == 1, layer.thisown

    def testBehaveWhenParentIsNotNull(self):
        """behave when parent (map) is not null"""
        self.initMap()
        layer = mapscript.layerObj(self.map)
        layer.name = "new"
        assert layer.refcount == 2, layer.refcount

    def testDummyClass(self):
        """basic refcounting for classObj"""
        clazz = mapscript.classObj()
        assert clazz.refcount == 1, clazz.refcount

    def testClassWithArgument(self):
        """classObj constructor with not null layer"""
        self.initMap()
        clazz = mapscript.classObj(self.map.getLayer(0))
        assert clazz.refcount == 2, clazz.refcount

    def testClassGetter(self):
        """classObj getter"""
        self.initMap()
        clazz = self.map.getLayer(1).getClass(0)
        assert clazz.refcount == 2, clazz.refcount

    def testClassClone(self):
        """classObj clone"""
        self.initMap()
        clazz = self.map.getLayer(1).getClass(0)
        assert clazz.refcount == 2, clazz.refcount
        clone = clazz.clone()
        assert clone.refcount == 1, clone.refcount

    def testClassInsert(self):
        """classObj insert at end"""
        self.initMap()
        clazz = mapscript.classObj()
        clazz.minscaledenom = 666
        assert clazz.refcount == 1, clazz.refcount
        idx = self.map.getLayer(1).insertClass(clazz)
        assert clazz.refcount == 2, clazz.refcount
        assert self.map.getLayer(1).getClass(idx).refcount == 3, (
            self.map.getLayer(1).getClass(idx).refcount
        )
        assert self.map.getLayer(1).getClass(idx).minscaledenom == 666, (
            self.map.getLayer(1).getClass(idx).minscaledenom
        )

    def testClassInsertAtMiddle(self):
        """classObj insert at pos. 1"""
        self.initMap()
        clazz = mapscript.classObj()
        clazz.minscaledenom = 666
        assert clazz.refcount == 1, clazz.refcount
        idx = self.map.getLayer(1).insertClass(clazz, 1)
        assert idx == 1, idx
        assert clazz.refcount == 2, clazz.refcount
        assert self.map.getLayer(1).getClass(idx).refcount == 3, (
            self.map.getLayer(1).getClass(idx).refcount
        )
        assert self.map.getLayer(1).getClass(idx).thisown, (
            self.map.getLayer(1).getClass(idx).thisown
        )
        assert self.map.getLayer(1).getClass(idx).minscaledenom == 666, (
            self.map.getLayer(1).getClass(idx).minscaledenom
        )

    def testRemoveFirstClass(self):
        """removal of first class"""
        self.initMap()
        clazz = self.map.getLayerByName("POLYGON").removeClass(0)
        assert clazz is not None
        assert clazz.thisown == 1
        assert clazz.refcount == 1, clazz.refcount
        self.map.draw()

    def testMapClone(self):
        """cloning a mapObj"""
        self.initMap()
        clone = self.map.clone()
        assert clone.refcount == 1, clone.refcount

    def testStyleGetter(self):
        """styleObj getter"""
        self.initMap()
        style = self.map.getLayer(1).getClass(0).getStyle(0)
        assert style.refcount == 2, style.refcount
        assert style.thisown == 1, style.thisown

    def testStyleConstructor(self):
        """styleObj Constructor"""
        self.initMap()
        style = mapscript.styleObj(self.map.getLayer(1).getClass(0))
        assert style.refcount == 2, style.refcount
        assert style.thisown == 1, style.thisown
        style = mapscript.styleObj()
        assert style.refcount == 1, style.refcount
        assert style.thisown == 1, style.thisown

    def testRemoveStyleAtBeginning(self):
        """styleObj remove first one"""
        self.initMap()
        style = mapscript.styleObj()
        assert style.refcount == 1, style.refcount
        assert style.thisown == 1, style.thisown
        self.map.getLayer(1).getClass(0).insertStyle(style)
        assert style.refcount == 2, style.refcount
        assert style.thisown == 1, style.thisown
        style = self.map.getLayer(1).getClass(0).removeStyle(0)
        assert style.refcount == 1, style.refcount
        assert style.thisown == 1, style.thisown

    def testInsertStyle(self):
        """styleObj Insert"""
        self.initMap()
        style = mapscript.styleObj()
        assert style.refcount == 1, style.refcount
        assert style.thisown == 1, style.thisown
        self.map.getLayer(1).getClass(0).insertStyle(style)
        assert style.refcount == 2, style.refcount
        assert style.thisown == 1, style.thisown

    def testCloneStyle(self):
        """styleObj Clone"""
        self.initMap()
        style = mapscript.styleObj()
        assert style.refcount == 1, style.refcount
        assert style.thisown == 1, style.thisown
        clone = style.clone()
        assert clone.refcount == 1, clone.refcount
        assert clone.thisown == 1, clone.thisown
        assert style.refcount == 1, style.refcount
        assert style.thisown == 1, style.thisown
        style = self.map.getLayer(1).getClass(0).getStyle(0)
        clone = style.clone()
        assert clone.refcount == 1, clone.refcount
        assert clone.thisown == 1, clone.thisown

    def testBasicSymbolRefcounting(self):
        """symbolObj refcounting"""
        self.initMap()
        symb = self.map.symbolset.getSymbol(0)
        assert symb.refcount == 2, symb.refcount
        assert symb.thisown == 1, symb.thisown
        symb = self.map.symbolset.getSymbolByName("home-png")
        assert symb.refcount == 2, symb.refcount
        assert symb.thisown == 1, symb.thisown
        symb2 = self.map.symbolset.getSymbolByName("home-png")
        assert symb2.refcount == 3, symb2.refcount
        assert symb2.thisown == 1, symb2.thisown

    def testSymbolAppendRemove(self):
        """appendSymbolObj refcounting"""
        self.initMap()
        symb = mapscript.symbolObj("ANEWSYMBOL")
        assert symb.refcount == 1, symb.refcount
        assert symb.thisown == 1, symb.thisown
        idx = self.map.symbolset.appendSymbol(symb)
        assert symb.refcount == 2, symb.refcount
        assert symb.thisown == 1, symb.thisown
        symb = self.map.symbolset.removeSymbol(idx)
        assert symb.refcount == 1, symb.refcount
        assert symb.thisown == 1, symb.thisown


if __name__ == "__main__":
    unittest.main()



#!/usr/bin/env python

"""
Extracts basic descriptive information from a Shapefile

Usage:

python shpinfo.py point.shp

"""
import os
import sys

import mapscript


def get_shapefile_object(sf_path):

    # make sure can access .shp file, create shapefileObj

    if os.access(sf_path, os.F_OK):
        sf_obj = mapscript.shapefileObj(sf_path)
    else:
        print("Can't access {}".format(sf_path))
        sys.exit(2)

    return sf_obj


def get_shapefile_details(sf_obj):

    # dictionary of shapefile types

    types = {
        mapscript.MS_SHAPEFILE_POINT: "Point",
        mapscript.MS_SHAPEFILE_ARC: "Line",
        mapscript.MS_SHAPEFILE_POLYGON: "Polygon",
        mapscript.MS_SHAPEFILE_MULTIPOINT: "Multipoint",
        mapscript.MS_SHP_POINTZ: "PointZ",
        mapscript.MS_SHP_ARCZ: "LineZ",
        mapscript.MS_SHP_POLYGONZ: "PolygonZ",
        mapscript.MS_SHP_MULTIPOINTZ: "MultipointZ",
        mapscript.MS_SHP_POINTM: "Multipoint",
        mapscript.MS_SHP_ARCM: "LineM",
        mapscript.MS_SHP_POLYGONM: "PolygonM",
        mapscript.MS_SHP_MULTIPOINTM: "MultipointM",
    }

    # print out basic information that is part of the shapefile object

    print("\tType: %s" % types[sf_obj.type])

    print(
        "\tBounds: (%f, %f) (%f, %f)"
        % (
            sf_obj.bounds.minx,
            sf_obj.bounds.miny,
            sf_obj.bounds.maxx,
            sf_obj.bounds.maxy,
        )
    )

    print("\tNumber of features: %i" % sf_obj.numshapes)


def get_dbf_details(sf_obj):

    # get DBF object

    dbf_obj = sf_obj.getDBF()

    # print out table characteristics

    print("\tNumber of records in DBF: %i" % dbf_obj.nRecords)
    print("\tNumber of fields: %i" % dbf_obj.nFields)
    print("")
    print("\t%-20s %12s %8s %10s" % ("Name", "Type", "Length", "Decimals"))
    print("\t-----------------------------------------------------")

    # print out field characteristics

    for idx in range(0, dbf_obj.nFields):
        print(
            "\t%-20s %12s %8d %10d"
            % (
                dbf_obj.getFieldName(idx),
                dbf_obj.getFieldType(idx),
                dbf_obj.getFieldWidth(idx),
                dbf_obj.getFieldDecimals(idx),
            )
        )


def main(sf_path):

    if not sf_path.lower().endswith(".shp"):
        sf_path += ".shp"

    sf_obj = get_shapefile_object(sf_path)

    print("Shapefile %s:" % sf_path)
    print("")

    get_shapefile_details(sf_obj)
    get_dbf_details(sf_obj)


def usage():
    """
    Display usage if program is used incorrectly
    """
    print("Syntax: %s <shapefile_path>" % sys.argv[0])
    sys.exit(2)


# make sure a filename argument is provided
if len(sys.argv) != 2:
    usage()

main(sys.argv[1])



# Project:  MapServer
# Purpose:  xUnit style Python mapscript tests of Map Styles
# Author:   Sean Gillies, sgillies@frii.com
#
# ===========================================================================
# Copyright (c) 2004, Sean Gillies
#
# Permission is hereby granted, free of charge, to any person obtaining a
# copy of this software and associated documentation files (the "Software"),
# to deal in the Software without restriction, including without limitation
# the rights to use, copy, modify, merge, publish, distribute, sublicense,
# and/or sell copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included
# in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
# OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
# THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
# DEALINGS IN THE SOFTWARE.
# ===========================================================================

import unittest

import mapscript

from .testing import MapTestCase


class DrawProgrammedStylesTestCase(MapTestCase):
    def testDrawPoints(self):
        """DrawProgrammedStylesTestCase.testDrawPoints: point drawing with styles works as advertised"""
        points = [
            mapscript.pointObj(-0.2, 51.6),
            mapscript.pointObj(0.0, 51.2),
            mapscript.pointObj(0.2, 51.6),
        ]
        colors = [
            mapscript.colorObj(255, 0, 0),
            mapscript.colorObj(0, 255, 0),
            mapscript.colorObj(0, 0, 255),
        ]
        img = self.map.prepareImage()
        layer = self.map.getLayerByName("POINT")
        # layer.draw(self.map, img)
        class0 = layer.getClass(0)
        for i in range(len(points)):
            style0 = class0.getStyle(0)
            style0.color = colors[i]
            # style0.color.pen = -4
            assert style0.color.toHex() == colors[i].toHex()
            points[i].draw(self.map, layer, img, 0, "foo")
        img.save("test_draw_points.png")


class NewStylesTestCase(MapTestCase):
    def testStyleConstructor(self):
        """a new style is properly initialized"""
        new_style = mapscript.styleObj()
        assert new_style.color.red == -1
        assert new_style.color.green == -1
        assert new_style.color.blue == -1

    def testCloneStyle(self):
        """check attributes of a cloned style"""
        new_style = mapscript.styleObj()
        new_style.color.setHex("#808080")
        clone = new_style.clone()
        assert clone.thisown == 1
        assert clone.color.toHex() == "#808080"

    def testStyleColorSettable(self):
        """a style can be set with a color tuple"""
        new_style = mapscript.styleObj()
        new_style.color.setRGB(1, 2, 3)
        assert new_style.color.red == 1
        assert new_style.color.green == 2
        assert new_style.color.blue == 3

    def testStyleBinding(self):
        """attribute binding can be set and get"""
        new_style = mapscript.styleObj()
        assert not new_style.getBinding(mapscript.MS_STYLE_BINDING_COLOR)
        new_style.setBinding(mapscript.MS_STYLE_BINDING_COLOR, "NEW_BINDING")
        assert new_style.getBinding(mapscript.MS_STYLE_BINDING_COLOR) == "NEW_BINDING"

    def testAppendNewStyle(self):
        """a new style can be appended properly"""
        p_layer = self.map.getLayerByName("POINT")
        class0 = p_layer.getClass(0)
        assert class0.numstyles == 2, class0.numstyles
        new_style = mapscript.styleObj()
        new_style.color.setRGB(0, 0, 0)
        new_style.setSymbolByName(self.map, "circle")
        new_style.size = 3
        index = class0.insertStyle(new_style)
        assert index == 2, index
        assert class0.numstyles == 3, class0.numstyles
        msimg = self.map.draw()
        assert msimg.thisown == 1
        filename = "testAppendNewStyle.png"
        fh = open(filename, "wb")
        msimg.write(fh)
        fh.close()

    def testAppendNewStyleOldWay(self):
        """NewStylesTestCase.testAppendNewStyleOldWay: a new style can be appended properly using old method"""
        p_layer = self.map.getLayerByName("POINT")
        class0 = p_layer.getClass(0)
        assert class0.numstyles == 2, class0.numstyles
        new_style = mapscript.styleObj(class0)
        assert new_style.thisown == 1, new_style.thisown
        new_style.color.setRGB(0, 0, 0)
        new_style.symbol = 1
        new_style.size = 3
        msimg = self.map.draw()
        filename = "testAppendNewStyleOldWay.png"
        fh = open(filename, "wb")
        msimg.write(fh)
        fh.close()

    def testInsertNewStyleAtIndex0(self):
        """NewStylesTestCase.testInsertNewStyleAtIndex0: a new style can be inserted ahead of all others"""
        l_layer = self.map.getLayerByName("LINE")
        class0 = l_layer.getClass(0)
        new_style = mapscript.styleObj()
        new_style.color.setRGB(255, 255, 0)
        new_style.symbol = 1
        new_style.size = 7
        index = class0.insertStyle(new_style, 0)
        assert index == 0, index
        assert class0.numstyles == 2, class0.numstyles
        msimg = self.map.draw()
        assert msimg.thisown == 1
        filename = "testInsertNewStyleAtIndex0.png"
        fh = open(filename, "wb")
        msimg.write(fh)
        fh.close()

    def testRemovePointStyle(self):
        """NewStylesTestCase.testRemovePointStyle: a point style can be properly removed"""
        p_layer = self.map.getLayerByName("POINT")
        class0 = p_layer.getClass(0)
        rem_style = class0.removeStyle(1)
        assert rem_style is not None
        assert class0.numstyles == 1, class0.numstyles
        msimg = self.map.draw()
        filename = "testRemovePointStyle.png"
        msimg.save(filename)

    def testModifyMultipleStyle(self):
        """NewStylesTestCase.testModifyMultipleStyle: multiple styles can be modified"""
        p_layer = self.map.getLayerByName("POINT")
        class0 = p_layer.getClass(0)
        style1 = class0.getStyle(1)
        style1.color.setRGB(255, 255, 0)
        msimg = self.map.draw()
        filename = "testModifyMultiplePointStyle.png"
        msimg.save(filename)

    def testInsertStylePastEnd(self):
        """NewStylesTestCase.testInsertStylePastEnd: inserting a style past the end of
        the list raises the proper error"""
        p_layer = self.map.getLayerByName("POINT")
        class0 = p_layer.getClass(0)
        new_style = mapscript.styleObj()
        self.assertRaises(
            mapscript.MapServerChildError, class0.insertStyle, new_style, 6
        )

    def testInsertNULLStyle(self):
        """attempt to insert a NULL style should raise error"""
        p_layer = self.map.getLayerByName("POINT")
        class0 = p_layer.getClass(0)
        self.assertRaises(mapscript.MapServerChildError, class0.insertStyle, None)

    def testConvertToString(self):
        """ensure styles can be created and output to strings
        in a round trip"""
        input_string = """
            STYLE
                SIZE 7.25
                COLOR 255 0 0
                OFFSET 10.5 20.75
            END
        """

        new_style = mapscript.fromstring(input_string)
        assert new_style.size == 7.25
        assert new_style.color.red == 255
        assert new_style.color.green == 0
        assert new_style.color.blue == 0

        assert new_style.offsetx == 10.5
        assert new_style.offsety == 20.75

        output_string = new_style.convertToString()
        new_style2 = mapscript.fromstring(output_string)

        # ensure attributes are output as doubles
        assert new_style.size == 7.25
        assert new_style2.offsetx == 10.5
        assert new_style2.offsety == 20.75

    def testPattern(self):
        """See https://github.com/mapserver/mapserver/issues/4943"""

        si = mapscript.styleObj()
        assert si.pattern == ()
        assert si.patternlength == 0

    def testPattern2(self):

        si = mapscript.styleObj()
        si.pattern = [2.0, 3, 4]
        assert si.pattern == (2.0, 3.0, 4.0)
        assert si.patternlength == 3

    def testPattern3(self):
        """a pattern must have at least 2 elements"""

        si = mapscript.styleObj()
        exception = None
        try:
            si.pattern = [1.0]
        except Exception:
            exception = True
        assert exception is True

    def testPattern4(self):
        """a pattern can have a max of 10 elements
        This is set in mapsymbol.h with #define MS_MAXPATTERNLENGTH 10"""

        si = mapscript.styleObj()
        exception = None
        try:
            si.pattern = [i for i in range(11)]
        except Exception:
            exception = True
        assert exception is True

    def testPattern5(self):
        """pattern length is read-only"""
        si = mapscript.styleObj()
        exception = None
        try:
            si.patternlength = 0
        except Exception:
            exception = True
        assert exception is True


class BrushCachingTestCase(MapTestCase):
    def testDrawMapWithSecondPolygon(self):
        """draw a blue polygon and a red polygon"""
        p = self.map.getLayerByName("POLYGON")
        ip = mapscript.layerObj(self.map)
        ip.type = mapscript.MS_LAYER_POLYGON
        ip.status = mapscript.MS_DEFAULT
        c0 = mapscript.classObj(ip)

        # turn off first polygon layer's color
        p.getClass(0).getStyle(0).color.setRGB(-1, -1, -1)

        # copy this style to inline polygon layer, then change outlinecolor
        c0.insertStyle(p.getClass(0).getStyle(0))
        st0 = c0.getStyle(0)
        st0.outlinecolor.setRGB(255, 0, 0)

        # pull out the first feature from polygon layer, shift it
        # and use this as an inline feature in new layer
        p.open()
        p.template = "FAKE"  # set a template so the layer can be queried
        p.queryByIndex(self.map, -1, 0, mapscript.MS_TRUE)
        res = p.getResult(0)
        s0 = p.getShape(res)
        p.close()
        r0 = s0.bounds
        r1 = mapscript.rectObj(
            r0.minx - 0.1, r0.miny - 0.1, r0.maxx - 0.1, r0.maxy - 0.1
        )
        s1 = r1.toPolygon()

        ip.addFeature(s1)
        img = self.map.draw()
        img.save("test_drawmapw2ndpolygon.png")


if __name__ == "__main__":
    unittest.main()



# Project:  MapServer
# Purpose:  xUnit style Python mapscript tests of Symbol
# Author:   Sean Gillies, sgillies@frii.com
#
# ===========================================================================
# Copyright (c) 2004, Sean Gillies
#
# Permission is hereby granted, free of charge, to any person obtaining a
# copy of this software and associated documentation files (the "Software"),
# to deal in the Software without restriction, including without limitation
# the rights to use, copy, modify, merge, publish, distribute, sublicense,
# and/or sell copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included
# in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
# OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
# THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
# DEALINGS IN THE SOFTWARE.
# ===========================================================================

import unittest
from io import BytesIO

import mapscript

from .testing import HOME_IMAGE, XMARKS_IMAGE, MapTestCase


class SymbolTestCase(unittest.TestCase):
    def testConstructor(self):
        """create new instance of symbolObj"""
        symbol = mapscript.symbolObj("test")
        assert symbol.name == "test"
        assert symbol.thisown == 1

    def testConstructorImage(self):
        """create new instance of symbolObj from an image"""
        symbol = mapscript.symbolObj("xmarks", XMARKS_IMAGE)
        assert symbol.name == "xmarks"
        assert symbol.type == mapscript.MS_SYMBOL_VECTOR  # was MS_SYMBOL_PIXMAP!
        format = mapscript.outputFormatObj("AGG/PNG")
        img = symbol.getImage(format)
        img.save("sym-%s.%s" % (symbol.name, img.format.extension))


class DynamicGraphicSymbolTestCase:
    def setUp(self):
        MapTestCase.setUp(self)
        with open(HOME_IMAGE, "rb") as fh:
            s = BytesIO(fh.read())
        symb_img = mapscript.imageObj(s)
        self.h_symbol = mapscript.symbolObj("house")
        self.h_symbol.type = mapscript.MS_SYMBOL_PIXMAP
        self.h_symbol.setImage(symb_img)
        with open(XMARKS_IMAGE, "rb") as fh:
            s = BytesIO(fh.read())
        symb_img = mapscript.imageObj(s)
        self.x_symbol = mapscript.symbolObj("xmarks")
        self.x_symbol.type = mapscript.MS_SYMBOL_PIXMAP
        self.x_symbol.setImage(symb_img)

    def testSetPCTImage(self):
        """set image of new symbolObj"""
        assert self.h_symbol.name == "house"
        assert self.h_symbol.type == mapscript.MS_SYMBOL_PIXMAP
        format = mapscript.outputFormatObj("AGG/PNG")
        format.transparent = mapscript.MS_ON
        img = self.h_symbol.getImage(format)
        img.save("set-%s.%s" % (self.h_symbol.name, img.format.extension))

    def testDrawSetPCTImage(self):
        """draw a map using the set image symbol"""
        symbol_index = self.map.symbolset.appendSymbol(self.h_symbol)
        assert symbol_index == 4, symbol_index
        num = self.map.symbolset.numsymbols
        assert num == 5, num
        inline_layer = self.map.getLayerByName("INLINE")
        s = inline_layer.getClass(0).getStyle(0)
        s.symbol = symbol_index
        s.size = -1  # pixmap's own size
        img = self.map.draw()
        img.save("testDrawSetPCTImage.%s" % (img.format.extension))

    def testSetRGBAImage(self):
        """set image of new symbolObj"""
        assert self.x_symbol.name == "xmarks"
        assert self.x_symbol.type == mapscript.MS_SYMBOL_PIXMAP
        format = mapscript.outputFormatObj("AGG/PNG")
        img = self.x_symbol.getImage(format)
        img.save("set-%s.%s" % (self.x_symbol.name, img.format.extension))

    def testDrawSetRGBAImage(self):
        """draw a map using the set image symbol"""
        symbol_index = self.map.symbolset.appendSymbol(self.x_symbol)
        inline_layer = self.map.getLayerByName("INLINE")
        s = inline_layer.getClass(0).getStyle(0)
        s.symbol = symbol_index
        s.size = -1  # pixmap's own size
        self.map.selectOutputFormat("PNG24")
        img = self.map.draw()
        img.save("testDrawSetRGBAImage.%s" % (img.format.extension))


class MapSymbolTestCase(MapTestCase):
    def testGetPoints(self):
        """get symbol points as line and test coords"""
        symbol = self.map.symbolset.getSymbol(1)
        assert symbol.name == "circle"
        line = symbol.getPoints()
        assert line.numpoints == 1, line.numpoints
        pt = self.getPointFromLine(line, 0)
        self.assertPointsEqual(pt, mapscript.pointObj(1.0, 1.0))

    def testSetPoints(self):
        """add lines of points to an existing symbol"""
        symbol = self.map.symbolset.getSymbol(1)
        assert symbol.name == "circle"
        line = mapscript.lineObj()
        self.addPointToLine(line, mapscript.pointObj(2.0, 2.0))
        self.addPointToLine(line, mapscript.pointObj(3.0, 3.0))
        assert symbol.setPoints(line) == 2
        assert symbol.numpoints == 2
        line = symbol.getPoints()
        assert line.numpoints == 2, line.numpoints
        pt = self.getPointFromLine(line, 1)
        self.assertPointsEqual(pt, mapscript.pointObj(3.0, 3.0))

    def testSetStyle(self):
        """expect success after setting an existing symbol's style"""
        symbol = self.map.symbolset.getSymbol(1)
        # default MapScript API setPattern has been extended to allow lists
        # assert symbol.setPattern(0, 1) == mapscript.MS_SUCCESS
        symbol.pattern = [0, 1]
        assert symbol.pattern == [0, 1]

    def testRGBASymbolInPNG24(self):
        """draw a RGBA PNG pixmap on PNG canvas"""
        self.map.setImageType("PNG24")
        # self.map.getLayerByName('INLINE-PIXMAP-RGBA').status \
        #    == mapscript.MS_DEFAULT
        img = self.map.draw()
        img.save("pixmap-rgba-24.png")

    def testRGBASymbolInJPEG(self):
        """draw a RGBA PNG pixmap on JPEG canvas"""
        self.map.setImageType("JPEG")
        # self.map.getLayerByName('INLINE-PIXMAP-RGBA').status \
        #    == mapscript.MS_DEFAULT
        img = self.map.draw()
        img.save("pixmap-rgba.jpg")


if __name__ == "__main__":
    unittest.main()



# Project:  MapServer
# Purpose:  xUnit style Python mapscript tests of OWS Requests
# Author:   Sean Gillies, sgillies@frii.com
#
# ===========================================================================
# Copyright (c) 2004, Sean Gillies
#
# Permission is hereby granted, free of charge, to any person obtaining a
# copy of this software and associated documentation files (the "Software"),
# to deal in the Software without restriction, including without limitation
# the rights to use, copy, modify, merge, publish, distribute, sublicense,
# and/or sell copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included
# in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
# OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
# THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
# DEALINGS IN THE SOFTWARE.
# ===========================================================================

import unittest
import xml.dom.minidom

import mapscript

from .testing import MapTestCase


class OWSRequestTestCase(MapTestCase):
    def testInit(self):
        """OWSRequestTestCase.testInit: OWS initializer works right"""
        request = mapscript.OWSRequest()
        request.setParameter("BBOX", "-0.3, 51.2, 0.3, 51.8")
        assert request.NumParams == 1
        assert request.getName(0) == "BBOX"
        assert request.getValue(0) == "-0.3, 51.2, 0.3, 51.8"

    def testGetParameter(self):
        """OWSRequestTestCase.testGetParameter: OWS can get request parameters by index"""
        request = mapscript.OWSRequest()
        request.setParameter("foo", "bar")
        assert request.getValue(0) == "bar"

    def testGetParameterByName(self):
        """OWSRequestTestCase.testGetParameterByName: OWS can get request parameters by name"""
        request = mapscript.OWSRequest()
        request.setParameter("foo", "bar")
        assert request.getValueByName("Foo") == "bar"

    def testResetParam(self):
        """OWSRequestTestCase.testResetParam: OWS can reset parameters by name"""
        request = mapscript.OWSRequest()
        request.setParameter("foo", "bar")
        assert request.NumParams == 1
        request.setParameter("Foo", "bra")
        assert request.NumParams == 1
        assert request.getValue(0) == "bra"

    def testLoadWMSRequest(self):
        """OWSRequestTestCase.testLoadWMSRequest: OWS can load a WMS request"""
        request = mapscript.OWSRequest()
        request.setParameter("REQUEST", "GetMap")
        request.setParameter("VERSION", "1.1.0")
        request.setParameter("FORMAT", "image/png")
        request.setParameter("LAYERS", "POINT")
        request.setParameter("BBOX", "-0.30, 51.20, 0.30, 51.80")
        request.setParameter("SRS", "EPSG:4326")
        request.setParameter("HEIGHT", "60")
        request.setParameter("WIDTH", "60")
        request.setParameter("STYLES", "")
        for i in range(self.map.numlayers):
            self.map.getLayer(i).status = mapscript.MS_ON
        status = self.map.loadOWSParameters(request)
        assert status == mapscript.MS_SUCCESS, status
        self.assertEqual(self.map.height, 60)
        self.assertEqual(self.map.width, 60)
        self.assertEqual(self.map.getProjection(), "+init=epsg:4326")
        # MapServer extents are from middle of the pixel
        self.assertAlmostEqual(self.map.extent.minx, -0.295)
        self.assertAlmostEqual(self.map.extent.miny, 51.205)
        self.assertAlmostEqual(self.map.extent.maxx, 0.295)
        self.assertAlmostEqual(self.map.extent.maxy, 51.795)
        img = self.map.draw()
        img.save("test_load_ows_request.png")

    def testWFSPostRequest(self):
        """OWSRequestTestCase.testLoadWMSRequest: OWS can POST a WFS request"""

        self.map.web.metadata.set("ows_onlineresource", "http://dummy.org/")
        request = mapscript.OWSRequest()
        request.contenttype = "application/xml"

        post_data = """<wfs:GetFeature xmlns:wfs="http://www.opengis.net/wfs" xmlns:ogc="http://www.opengis.net/ogc" service="WFS"
        version="1.1.0" xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance">
        <wfs:Query typeName="*:POINT" xmlns:feature="http://www.openplans.org/topp">
            <ogc:Filter>
                <ogc:PropertyIsEqualTo>
                    <ogc:PropertyName>FID</ogc:PropertyName>
                    <ogc:Literal>1</ogc:Literal>
                </ogc:PropertyIsEqualTo>
            </ogc:Filter>
        </wfs:Query>
        </wfs:GetFeature>
        """

        qs = ""  # additional parameters can be passed via the querystring
        request.loadParamsFromPost(post_data, qs)
        mapscript.msIO_installStdoutToBuffer()
        status = self.map.OWSDispatch(request)
        assert status == mapscript.MS_SUCCESS, status

        mapscript.msIO_stripStdoutBufferContentHeaders()
        result = mapscript.msIO_getStdoutBufferBytes()
        dom = xml.dom.minidom.parseString(result)
        assert len(dom.getElementsByTagName("ms:POINT")) == 1


if __name__ == "__main__":
    unittest.main()



# Project:  MapServer
# Purpose:  xUnit style Python mapscript tests of Point
# Author:   Sean Gillies, sgillies@frii.com
#
# ===========================================================================
# Copyright (c) 2004, Sean Gillies
#
# Permission is hereby granted, free of charge, to any person obtaining a
# copy of this software and associated documentation files (the "Software"),
# to deal in the Software without restriction, including without limitation
# the rights to use, copy, modify, merge, publish, distribute, sublicense,
# and/or sell copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included
# in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
# OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
# THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
# DEALINGS IN THE SOFTWARE.
# ===========================================================================

import unittest

import mapscript

have_image = False

try:
    from PIL import Image

    have_image = True
except ImportError:
    pass


class PointObjTestCase(unittest.TestCase):
    def testPointObjConstructorNoArgs(self):
        """point can be created with no arguments"""
        p = mapscript.pointObj()
        self.assertAlmostEqual(p.x, 0.0)
        self.assertAlmostEqual(p.y, 0.0)

    def testPointObjConstructorArgs(self):
        """point can be created with arguments"""
        p = mapscript.pointObj(1.0, 1.0)
        self.assertAlmostEqual(p.x, 1.0)
        self.assertAlmostEqual(p.y, 1.0)

    def testSetXY(self):
        """point can have its x and y reset"""
        p = mapscript.pointObj()
        p.setXY(1.0, 1.0)
        self.assertAlmostEqual(p.x, 1.0)
        self.assertAlmostEqual(p.y, 1.0)
        if hasattr(p, "m"):
            self.assertAlmostEqual(p.m, -2e38)

    def testSetXYM(self):
        """point can have its x and y reset (with m value)"""
        p = mapscript.pointObj()
        p.setXY(1.0, 1.0, 1.0)
        self.assertAlmostEqual(p.x, 1.0)
        self.assertAlmostEqual(p.y, 1.0)
        if hasattr(p, "m"):
            self.assertAlmostEqual(p.m, 1.0)

    def testSetXYZ(self):
        """point can have its x, y, z reset (with m value)"""
        p = mapscript.pointObj()
        p.setXYZ(1.0, 2.0, 3.0, 4.0)
        self.assertAlmostEqual(p.x, 1.0)
        self.assertAlmostEqual(p.y, 2.0)
        if hasattr(p, "z") and hasattr(p, "m"):
            self.assertAlmostEqual(p.z, 3.0)
            self.assertAlmostEqual(p.m, 4.0)

    def testPointDraw(self):
        """Can create a point, add to a layer, and draw it directly"""

        map_string = """
        MAP
            EXTENT 0 0 90 90
            SIZE 500 500

            SYMBOL
            NAME "circle"
            TYPE ELLIPSE
            POINTS 1 1 END
            FILLED true
            END

            LAYER
            NAME "punkt"
            STATUS ON
            TYPE POINT
            END

        END"""

        test_map = mapscript.fromstring(map_string)
        layer = test_map.getLayerByName("punkt")
        cls = mapscript.classObj()
        style = mapscript.styleObj()
        style.outlinecolor.setHex("#00aa00ff")
        style.size = 10
        style.setSymbolByName(test_map, "circle")

        cls.insertStyle(style)
        class_idx = layer.insertClass(cls)
        point = mapscript.pointObj(45, 45)

        img = test_map.prepareImage()
        point.draw(test_map, layer, img, class_idx, "test")

        filename = "testDrawPoint.png"
        with open(filename, "wb") as fh:
            img.write(fh)

        if have_image:
            pyimg = Image.open(filename)
            assert pyimg.size == (500, 500)

    def testPoint__str__(self):
        """return properly formatted string"""
        p = mapscript.pointObj(1.0, 1.0)
        if hasattr(p, "z"):
            p_str = "{ 'x': %.16g, 'y': %.16g, 'z': %.16g }" % (p.x, p.y, p.z)
        else:
            p_str = "{ 'x': %.16g, 'y': %.16g }" % (p.x, p.y)
        assert str(p) == p_str, str(p)

    def testPointToString(self):
        """return properly formatted string in toString()"""
        p = mapscript.pointObj(1.0, 1.0, 0.002, 15.0)
        if hasattr(p, "z") and hasattr(p, "m"):
            p_str = "{ 'x': %.16g, 'y': %.16g, 'z': %.16g, 'm': %.16g }" % (
                p.x,
                p.y,
                p.z,
                p.m,
            )
        else:
            p_str = "{ 'x': %.16g, 'y': %.16g }" % (p.x, p.y)

        assert p.toString() == p_str, p.toString()

    def testPointGeoInterface(self):
        """return point using the  __geo_interface__ protocol"""
        p = mapscript.pointObj(1.0, 1.0, 0.002, 15.0)
        if hasattr(p, "z"):
            assert p.__geo_interface__ == {
                "type": "Point",
                "coordinates": (1.0, 1.0, 0.002),
            }
        else:
            assert p.__geo_interface__ == {"type": "Point", "coordinates": (1.0, 1.0)}


if __name__ == "__main__":
    unittest.main()



#!/usr/bin/env python

"""
Dump the contents of the passed in Shapefile

Usage:

python shpdump.py polygon.shp

"""

import os
import sys

import mapscript


def plural(x):
    """
    Returns an 's' if plural

    Useful in print statements to avoid something like 'point(s)'
    """
    if x > 1:
        return "s"
    return ""


def get_shapefile_object(sf_path):

    # make sure can access .shp file, create shapefileObj

    if os.access(sf_path, os.F_OK):
        sf_obj = mapscript.shapefileObj(sf_path, -1)
    else:
        print("Can't access {}".format(sf_path))
        sys.exit(2)

    return sf_obj


def main(sf_path):

    if not sf_path.lower().endswith(".shp"):
        sf_path += ".shp"

    sf_obj = get_shapefile_object(sf_path)

    # create an empty Shapefile object

    s_obj = mapscript.shapeObj()

    # loop through each shape in the original Shapefile

    for i in range(sf_obj.numshapes):

        # get the object at index i
        sf_obj.get(i, s_obj)
        print("Shape %i has %i part%s" % (i, s_obj.numlines, plural(s_obj.numlines)))
        print(
            "Bounds (%f, %f) (%f, %f)"
            % (
                s_obj.bounds.minx,
                s_obj.bounds.miny,
                s_obj.bounds.maxx,
                s_obj.bounds.maxy,
            )
        )

        # loop through parts of each shape

        for j in range(s_obj.numlines):

            # get the jth part of the ith object

            part = s_obj.get(j)
            print(
                "Part %i has %i point%s" % (j, part.numpoints, plural(part.numpoints))
            )

            # loop through points in each part

            for k in range(part.numpoints):

                # get the kth point of the jth part of the ith shape

                point = part.get(k)
                print("%i: %f, %f" % (k, point.x, point.y))


def usage():
    """
    Display usage if program is used incorrectly
    """
    print("Syntax: %s <shapefile_path>" % sys.argv[0])
    sys.exit(2)


# make sure passing in filename argument
if len(sys.argv) != 2:
    usage()


sf_path = sys.argv[1]
main(sf_path)



# Project:  MapServer
# Purpose:  xUnit style Python mapscript tests of shapefileObj
# Author:   Sean Gillies, sgillies@frii.com
#
# ===========================================================================
# Copyright (c) 2004, Sean Gillies
#
# Permission is hereby granted, free of charge, to any person obtaining a
# copy of this software and associated documentation files (the "Software"),
# to deal in the Software without restriction, including without limitation
# the rights to use, copy, modify, merge, publish, distribute, sublicense,
# and/or sell copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included
# in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
# OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
# THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
# DEALINGS IN THE SOFTWARE.
# ===========================================================================

import os
import unittest

import mapscript

from .testing import TESTS_PATH


class AddShapeTestCase(unittest.TestCase):
    def testAddEmpty(self):
        """expect an error rather than segfault when adding an empty shape"""
        # See bug 1201
        sf = mapscript.shapefileObj("testAddDud.shp", 1)
        so = mapscript.shapeObj(mapscript.MS_SHAPE_POINT)
        self.assertRaises(mapscript.MapServerError, sf.add, so)

    def testGetDBFInfo(self):
        """Fetch dbf information from shapefile"""
        pth = os.path.join(TESTS_PATH, "polygon.shp")
        sf = mapscript.shapefileObj(pth)
        assert sf.getDBF() is not None, sf.getDBF()
        assert sf.getDBF().nFields == 2, sf.getDBF().nFields
        assert sf.getDBF().getFieldName(0) == "FID", sf.getDBF().getFieldName(0)
        assert sf.getDBF().getFieldName(1) == "FNAME", sf.getDBF().getFieldName(1)


if __name__ == "__main__":
    unittest.main()
